.686

.MODEL FLAT,STDCALL

OPTION	CASEMAP:NONE

INCLUDE board.inc
INCLUDE graphics.inc

;/*******************************************
; Data
;********************************************/

.DATA

ALIGN 8

Board1		    DWORD   0, 0
Board2		    DWORD   0, 0
BoardHeight     DWORD   BOARD_NBCOL DUP (0)
BoardHeightCopy DWORD   BOARD_NBCOL DUP (0)
BoardThreat     DWORD   BOARD_NBTHREAT DUP (0)
BoardNote       DWORD   0
BOARD_DATASIZE = $ - OFFSET Board1

ALIGN 4
BoardStruct BOARDSTRUCT <>

;/*******************************************
; Code
;********************************************/

.CODE

;/*************************************
;* Resume : Initialisation tableau
;**************************************/
BoardInit::

		push	edi

		mov		ecx, BOARD_DATASIZE
		xor		eax, eax
		mov		edi, OFFSET Board1
		rep		stosb
		mov		edi, OFFSET BoardStruct
		ASSUME	edi:PTR BOARDSTRUCT
		mov		[edi].Player, BOARD_PLAYER1
		mov		[edi].Status, BOARD_PLAYER1TURN

		pop		edi

		ret

;/*************************************
;* Resume : Renvoie une case
;* Params : EAX : numero de colonne
;* Params : ECX : numero de ligne
;* Retour : EAX : la case
;**************************************/

BoardGetSquare::

        mov     edx, eax
        xor     eax, eax
        bt      Board2[ecx], edx
        rcl     eax, 1
        bt      Board1[ecx], edx
        rcl     eax, 1

		ret

;/*************************************
;* Resume : Init d'une case
;* Params : EBX : player
;* Params : ECX : colonne
;* Params : EDX : hauteur
;* Retour : aucun
;**************************************/

BoardSetSquare::

        ;// set le bit 8*hauteur + colonne a partir de board1 + 8 * (player - 1)
        ;// cad le bit colonne a partir de 8*(player-1) + hauteur
        
        bts     Board1[8*ebx - 8 + edx], ecx 
        inc     BoardHeight[4*ecx]

BoardSetSquareAi::

        inc     BoardHeightCopy[4*ecx]
        call    BoardThreatSet

		ret

;/*************************************
;* Resume : Annulation coup precedent
;* Retour : EBX player ayant joue le coup
;* Retour : ECX colonne du coup
;**************************************/

BoardUnsetSquare::

        
        mov     edx, BoardHeightCopy[4*ecx]
        dec     edx
        dec     BoardHeightCopy[4*ecx]

        call    BoardThreatUnset

        ret


;/*************************************
;* Resume : Appelee apres un click sur une colonne
;* Params : EAX : numero de colonne
;* Retour : EAX : status du jeu apres le click
;**************************************/

BoardClick::
		
        push    ebx

        cmp     BoardHeight[4*eax], BOARD_NBROW
        je      ColFull

        mov     ebx, BoardStruct.Player
        mov     edx, BoardHeight[4*eax]
        mov     ecx, eax
        push    eax
        call    BoardSetSquare
        pop     eax
       
        call    GraphicsDrawClickedSquare

        call    BoardGetStatus

ColFull :

        mov     eax, BoardStruct.Status

        pop     ebx

		ret


;/*************************************
;* Resume : Calcule le status du jeu
;* Retour : EAX, BoardStruct.Status : status du jeu
;**************************************/

BoardGetStatus::

        push    esi
        push    edi

        mov     esi, OFFSET BoardStruct
        ASSUME  esi:PTR_BOARDSTRUCT

        ;// estime le damier
        call    BoardTestWin
        jge     Win

        ;// teste draw
        mov     ecx, BOARD_NBCOL + 1
        mov     eax, BOARD_NBROW
        mov     edi, OFFSET BoardHeight
        repe    scasd
        jecxz   Draw

NotDraw :

        ;// joueur suivant
        mov     eax, [esi].Status
        xor     eax, 011b
        mov     [esi].Player, eax
        mov     [esi].Status, eax

@@ :

        pop     edi
        pop     esi

        ret

Win :

        add     [esi].Status, 2
        jmp     @B

Draw :

        ;// draw
        mov     [esi].Status, BOARD_DRAW

        jmp     @B

        ASSUME  esi:NOTHING

;/*************************************
;* Resume : Teste si un joueur a gagne
;* Retour : FLAG GE SET
;**************************************/

BoardTestWin::

        mov     eax, BoardNote[0]

        cdq
        xor     eax, edx
        sub     eax, edx

        cmp     eax, ABS_WIN_NOTE
        jge     @F

        mov     eax, BoardNote[4]
        cdq
        xor     eax, edx
        sub     eax, edx
        cmp     eax, ABS_WIN_NOTE

@@:
        
        ret


;/*************************************
;* Resume : Met a jour les menaces apres un coup
;* Params : EBX : joueur
;* Params : ECX : colonne
;* Params : EDX : hauteur
;**************************************/

BoardThreatSet::

        push    esi
        push    edi

        ;// esi = pointeur sur offset
        lea     eax, [8*edx+ecx]
        
        xor     ecx, ecx
        xor     edi, edi

        mov     esi, ThreatOffsetTable[4*eax]

        POFFSET TEXTEQU <esi>
        
@@ :
        ;// ax = (num_threat, off_threat)
        movzx   eax, WORD PTR [POFFSET]
        mov     edx, eax

        and     eax, 0FFh
        shr     edx, 8

        ;// edx = index threat
        ;// eax = offset dans threat
        lea     ecx, [2*eax + ebx - 1]
        add     POFFSET, 2
        mov     eax, BoardThreat[4*edx]
        bts     eax, ecx        
        add     edi, NoteTable[4*eax]
        mov     BoardThreat[4*edx], eax
        
        cmp     WORD PTR [POFFSET], 0FFFFh
        jne     @B

        mov     BoardNote[0], edi

        pop     edi
        pop     esi

        ret

;/*************************************
;* Resume : Met a jour les menaces apres l'annulation d'un coup
;* Params : EBX : joueur
;* Params : ECX : colonne
;* Params : EDX : hauteur
;**************************************/

BoardThreatUnset::

        push    esi
        push    edi

        ;// esi = pointeur sur offset
        lea     eax, [8*edx+ecx]
        
        xor     ecx, ecx
        xor     edi, edi

        mov     esi, ThreatOffsetTable[4*eax]

        POFFSET TEXTEQU <esi>
        
@@ :
        ;// ax = (num_threat, off_threat)
        movzx   eax, WORD PTR [POFFSET]
        mov     edx, eax

        and     eax, 0FFh
        shr     edx, 8

        ;// edx = index threat
        ;// eax = offset dans threat
        lea     ecx, [2*eax + ebx - 1]
        add     POFFSET, 2
        mov     eax, BoardThreat[4*edx]
        btr     eax, ecx        
        add     edi, NoteTable[4*eax]
        mov     BoardThreat[4*edx], eax
        
        cmp     WORD PTR [POFFSET], 0FFFFh
        jne     @B

        mov     BoardNote[0], edi

        pop     edi
        pop     esi

        ret


;/*************************************
;* Resume : Estime le damier pour le joueur courant
;* Params : EBX : oppose du joueur pour qui estimer le damier
;* Retour : EAX : note donnee, > abs(max) si coup gagnant
;**************************************/

BoardEstimate ::

        mov     eax, BoardNote[0]

        ret

END