;/*******************************************
; Graphics part of the game
;********************************************/

.686
.MODEL  FLAT, STDCALL
OPTION CASEMAP:NONE

;/*******************************************
; INCLUDE
;********************************************/

INCLUDE windows.inc
INCLUDE gdi32.inc
INCLUDE user32.inc

INCLUDE graphics.inc
INCLUDE board.inc
INCLUDE main.inc

;/*******************************************
; CONST
;********************************************/

SQUARE1_COLOR = 0FFFFh		;// yellow
SQUARE2_COLOR = 0FFh		;// red
WHITE_COLOR = 000FFFFFFh	;// white
BACKGROUND_COLOR = 0FF0000h; ;// blue

;/*******************************************
; DATA
;********************************************/

.DATA

ALIGN 4

;// Brush
NB_BRUSH = 4
ID_BRUSH_WHITE = 4*0
ID_BRUSH_SQUARE1 = 4*1
ID_BRUSH_SQUARE2 = 4*2
ID_BRUSH_BACKGROUND = 4*3

Brush HBRUSH NB_BRUSH DUP (0)
BrushColor DWORD WHITE_COLOR, SQUARE1_COLOR, SQUARE2_COLOR, BACKGROUND_COLOR

GraphicsStruct GRAPHICSSTRUCT <>

;/*******************************************
; CODE
;********************************************/

.CODE

;/*************************************
;* Resume : Init les graphismes
;* Params : EAX : dc de l'ecran
;**************************************/

GraphicsInit::

		push	ebx
        push    esi

		mov		ebx, OFFSET GraphicsStruct
		ASSUME	ebx:PTR_GRAPHICSSTRUCT

		mov		[ebx].Hdc, eax
		call	GraphicsSize

		mov		esi, NB_BRUSH-1
@@:		
		push	BrushColor[4*esi]
		call	CreateSolidBrush
		mov		Brush[4*esi], eax
		dec		esi
		jns		@B

		pop     esi
        pop		ebx		

		ret


;/*************************************
;* Resume : Affiche tout le damier
;**************************************/

GraphicsPaint::

        push    ebp
        push    ebx
        push    edi
        push    esi

        X1      TEXTEQU <ebp>
        Y1      TEXTEQU <ebx>
        X2      TEXTEQU <edi>
        Y2      TEXTEQU <esi>

        ;// dessine le fond
		push	Brush[ID_BRUSH_BACKGROUND]
		push	OFFSET GraphicsStruct.ScreenLeft
		push	GraphicsStruct.Hdc
		call	FillRect

        ;// locales
        sub     esp, 4*2
        I       TEXTEQU <DWORD PTR [esp + 4]>
        J       TEXTEQU <DWORD PTR [esp]>

        ;// boucle d'affichage
        mov     I, BOARD_NBROW - 1
        mov     Y1, GraphicsStruct.OffsetY
        mov     Y2, GraphicsStruct.SquareHeight
        add     Y2, Y1

RowLoop :

        mov     X1, GraphicsStruct.OffsetX
        mov     X2, GraphicsStruct.SquareWidth
        add     X2, X1
        mov     J, 0

ColLoop :

        mov     eax, J
        mov     ecx, I
        call    BoardGetSquare

        push    Brush[4*eax]
        push    GraphicsStruct.Hdc
        call    SelectObject

        push    Y2
        push    X2
        push    Y1
        push    X1
        push	GraphicsStruct.Hdc
		call	Ellipse

        mov     X1, X2
        add     X2, GraphicsStruct.SquareWidth
        
        inc     J
        cmp     J, BOARD_NBCOL
        jb      ColLoop

        mov     Y1, Y2
        add     Y2, GraphicsStruct.SquareHeight

        dec     I
        jns     RowLoop

        add     esp, 4*2
        				
		pop		esi
		pop		edi
		pop		ebx
		pop		ebp

		ret

;/*************************************
;* Resume : Affiche une case
;* Params : EAX : colonne de la case
;**************************************/

GraphicsDrawClickedSquare ::

        push    ebx

        mov     ebx, OFFSET GraphicsStruct
        ASSUME  ebx:PTR_GRAPHICSSTRUCT
       
        push    eax

        mov     eax, BoardStruct.Player

        push    Brush[4*eax]
        push    [ebx].Hdc
        call    SelectObject

        pop     eax

        ;// eax = x2 = (col + 1) * SQUARE_WIDTH     
        ;// ecx = y2 = (NB_ROW - height[col] + 1) * SQUARE_HEIGHT
        mov     ecx, BOARD_NBROW + 1
        sub     ecx, BoardHeight[4*eax]
        inc     eax
        imul    eax, [ebx].SquareWidth
        imul    ecx, [ebx].SquareHeight       

        add     eax, [ebx].OffsetX
        add     ecx, [ebx].OffsetY

        push    ecx
        push    eax
        sub     ecx, [ebx].SquareHeight
        sub     eax, [ebx].SquareWidth  
        push    ecx
        push    eax
        push	[ebx].Hdc
		call	Ellipse

        pop     ebx

		ret

;/*************************************
;* Resume : Resize les graphismes
;**************************************/

GraphicsSize::

		push	ebx
		push	esi
		push	edi

		mov		ebx, OFFSET GraphicsStruct
		ASSUME	ebx:PTR_GRAPHICSSTRUCT

		sub		esp, SIZEOF(RECT)	

		push	esp
		push	MainStruct.Hwnd
		call	GetClientRect

		mov		esi, [esp].RECT.right
		mov		edi, [esp].RECT.bottom

		push	esp
		push	MainStruct.Hstatus
		call	GetClientRect

		sub		edi, [esp].RECT.bottom

		mov		[ebx].ScreenWidth, esi
		mov		[ebx].ScreenHeight, edi

        mov     eax, esi
        mov     ecx, BOARD_NBCOL
        xor     edx, edx
        div     ecx

        shr     edx, 1
        mov     [ebx].OffsetX, edx

        mov     [ebx].SquareWidth, eax

        mov     eax, edi
        mov     ecx, BOARD_NBROW
        xor     edx, edx
        div     ecx

        shr     edx, 1
        mov     [ebx].OffsetY, edx

        mov     [ebx].SquareHeight, eax

		add		esp, SIZEOF(RECT)

	    pop		edi
		pop		esi
		pop		ebx

		ret

;/*************************************
;* Resume : Libere les ressources graphiques
;**************************************/

GraphicsQuit::
       
        push	ebx

		mov		ebx, NB_BRUSH-1
@@:		
		push	Brush[4*ebx]
		call	DeleteObject
		dec		ebx
		jns		@B
        
		pop		ebx

		ret

END